from spaceone.core.error import *


class ERROR_NO_PLUGIN(ERROR_INVALID_ARGUMENT):
    _message = 'Plugin does not exists. (plugin_id = {plugin_id})'


class ERROR_INVALID_IMAGE_FORMAT(ERROR_INVALID_ARGUMENT):
    _message = 'Image format is <repository name>/<image name> in {name}'


class ERROR_INVALID_IMAGE_NAME_FORMAT(ERROR_INVALID_ARGUMENT):
    _message = 'Image name format is [a-zA-Z0-9\-]+ in {name}'


class ERROR_INVALID_IMAGE_LENGTH(ERROR_INVALID_ARGUMENT):
    _message = 'The length of image should be less than equal {length} in {name}'


class ERROR_INVALID_TEMPLATE_SCHEMA(ERROR_INVALID_ARGUMENT):
    _message = 'Template schema format(JSON SCHEMA) is invalid. (key = {key})'


class ERROR_NO_IMAGE_IN_REGISTRY(ERROR_UNKNOWN):
    _message = 'Image does not found. (registry_type = {registry_type}, image = {image})'


class ERROR_REGISTRY_SETTINGS(ERROR_UNKNOWN):
    _message = 'Registry settings are not ready. (registry_type = {registry_type})'



import abc
import requests
import logging
import boto3
from distutils.version import StrictVersion

from spaceone.core.connector import BaseConnector
from spaceone.repository.error import *

__all__ = ["DockerHubConnector", "AWSPrivateECRConnector", "HarborConnector"]

_LOGGER = logging.getLogger(__name__)
_DEFAULT_PAGE_SIZE = 10


class RegistryConnector(BaseConnector):
    @abc.abstractmethod
    def get_tags(self, registry_url, image):
        pass


class DockerHubConnector(RegistryConnector):
    def get_tags(self, registry_url, image):
        url = f"https://{registry_url}/v2/repositories/{image}/tags"

        response = requests.get(url)

        if response.status_code == 200:
            image_tags = []
            results = response.json().get("results", [])
            for tag in results:
                image_tags.append(tag["name"])

            return image_tags

        else:
            raise ERROR_NO_IMAGE_IN_REGISTRY(registry_type="DOCKER_HUB", image=image)


class HarborConnector(RegistryConnector):
    def get_tags(self, registry_url, image):
        base_url = self.config.get("base_url")
        verify = self.config.get("verify", True)

        headers = {"authorization": f'Basic {self.config["token"]}'}

        url = f"{base_url}/v2/{image}/tags/list"

        response = requests.get(url, headers=headers, verify=verify)

        if response.status_code == 200:
            image_tags = response.json().get("tags", [])
            image_tags.sort(key=StrictVersion, reverse=True)
            return image_tags
        else:
            _LOGGER.error(f"[get_tags] request error: {response.json()}")

        raise ERROR_NO_IMAGE_IN_REGISTRY(registry_type="HARBOR", image=image)


class AWSPrivateECRConnector(RegistryConnector):
    _DEFAULT_REGION_NAME = "ap-northeast-2"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        aws_access_key_id = self.config.get("aws_access_key_id")
        aws_secret_access_key = self.config.get("aws_secret_access_key")

        if not all([aws_access_key_id, aws_secret_access_key]):
            raise ERROR_CONNECTOR_CONFIGURATION(connector="AWSPrivateECRConnector")

        self.client = boto3.client(
            "ecr",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=self._DEFAULT_REGION_NAME,
        )

    def get_tags(self, registry_url, image):
        try:
            response = self.client.describe_images(
                repositoryName=image, registryId=self.config.get("account_id")
            )

            image_tags = []
            images_info = response.get("imageDetails", [])
            sorted_images_info = sorted(
                images_info, key=lambda k: k["imagePushedAt"], reverse=True
            )
            for image_info in sorted_images_info:
                image_tags.extend(image_info["imageTags"])

            return image_tags

        except Exception as e:
            _LOGGER.error(f"[get_tags] boto3 describe_image_tags error: {e}")
            raise ERROR_NO_IMAGE_IN_REGISTRY(
                registry_type="AWS_PRIVATE_ECR", image=image
            )


if __name__ == "__main__":
    docker_hub_conn = DockerHubConnector()
    tags = docker_hub_conn.get_tags(
        "registry.hub.docker.com", "cloudforet/repository", {}
    )



import functools
from spaceone.api.repository.v1 import plugin_pb2, repository_pb2
from spaceone.core import config
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

__all__ = ["PluginInfo", "PluginsInfo", "VersionsInfo"]


def PluginInfo(plugin_info: dict, minimal=False):
    info = {
        "plugin_id": plugin_info.get("plugin_id"),
        "name": plugin_info.get("name"),
        "state": plugin_info.get("state"),
        "image": plugin_info.get("image"),
        "registry_type": plugin_info.get("registry_type"),
        "resource_type": plugin_info.get("resource_type"),
        "provider": plugin_info.get("provider"),
    }
    if not minimal:
        registry_info = config.get_global("REGISTRY_INFO", {}).get(
            plugin_info.get("registry_type")
        )

        info.update(
            {
                "registry_config": change_struct_type(registry_info),
                "capability": change_struct_type(plugin_info.get("capability")),
                "tags": change_struct_type(plugin_info.get("tags")),
                "labels": change_list_value_type(plugin_info.get("labels")),
                "domain_id": plugin_info.get("domain_id"),
                "created_at": utils.datetime_to_iso8601(plugin_info.get("created_at")),
                "updated_at": utils.datetime_to_iso8601(plugin_info.get("updated_at")),
            }
        )

        if "registry_url" in plugin_info:
            info["registry_url"] = plugin_info["registry_url"]
        else:
            info["registry_url"] = registry_info.get("url")

        if repository_info := plugin_info.get("repository_info"):
            info.update(
                {
                    "repository_info": repository_pb2.RepositoryInfo(
                        repository_id=repository_info.get("repository_id"),
                        name=repository_info.get("name"),
                        repository_type=repository_info.get("repository_type"),
                    )
                }
            )

    return plugin_pb2.PluginInfo(**info)


def PluginsInfo(plugins_info, total_count, **kwargs):
    return plugin_pb2.PluginsInfo(
        results=list(map(functools.partial(PluginInfo, **kwargs), plugins_info)),
        total_count=total_count,
    )


def VersionsInfo(version_list):
    return plugin_pb2.VersionsInfo(results=version_list, total_count=len(version_list))
